import os
from Bio import SeqIO


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if each_element.isalpha() is True:
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


sam_file        = '/Users/songweizhi/Desktop/step_1_unlinked_marker_genes_ends_3000bp.sam'
ref_in          = '/Users/songweizhi/Desktop/step_1_unlinked_marker_genes_ends_3000bp.fasta'
max_gap_to_end  = 300
minCigarM       = 50
end_seq_len     = 3000


# get ref seqs subset
ref_subset_len_dict = {}
for ref_seq in SeqIO.parse(ref_in, 'fasta'):
    ref_seq_id = ref_seq.id
    ref_seq_len = len(ref_seq.seq)
    if ref_seq_len < end_seq_len * 3:
        ref_subset_len_dict[ref_seq_id] = ref_seq_len
    else:
        ref_seq_left_end_id = '%s_l' % ref_seq_id
        ref_seq_right_end_id = '%s_r' % ref_seq_id
        ref_seq_left_end = ref_seq.seq[:end_seq_len]
        ref_seq_right_end = ref_seq.seq[-end_seq_len:]


# parse sam file
all_mapped_reads = set()
qualified_reads_dict = {}
qualified_reads_to_ref_dict = {}
for each_line in open(sam_file):
    if not each_line.startswith('@'):
        each_line_split = each_line.strip().split('\t')
        cigar = each_line_split[5]

        if cigar != '*':
            read_id = each_line_split[0]
            read_id_base = '.'.join(read_id.split('.')[:-1])
            read_strand = read_id.split('.')[-1]
            ref_id = each_line_split[2]
            ref_pos = int(each_line_split[3])
            cigar_splitted = cigar_splitter(cigar)
            all_mapped_reads.add(read_id)
            qualified_mapping = False

            if ref_id[-2:] == '_l':
                if ref_pos <= max_gap_to_end:
                    if (len(cigar_splitted) == 1) and (cigar[-1] == 'M') and (int(cigar[:-1]) >= minCigarM):
                        qualified_mapping = True


                    elif (len(cigar_splitted) == 2) and (cigar_splitted[-1][-1] == 'M') and (
                            cigar_splitted[0][-1] == 'S') and (int(cigar_splitted[-1][:-1]) >= minCigarM) and (
                            ref_pos == 1):
                        qualified_mapping = True


            elif ref_id[-2:] == '_r':

                if (len(cigar_splitted) == 1) and (cigar[-1] == 'M') and (int(cigar[:-1]) >= minCigarM):
                    ref_pos_end = ref_pos + int(cigar[:-1])
                    if (end_seq_len - ref_pos_end) <= max_gap_to_end:
                        qualified_mapping = True

                elif (len(cigar_splitted) == 2) and (cigar_splitted[0][-1] == 'M') and (
                        cigar_splitted[1][-1] == 'S') and (int(cigar_splitted[0][:-1]) >= minCigarM):
                    if (ref_pos + int(cigar_splitted[0][:-1]) - 1) == end_seq_len:
                        qualified_mapping = True

            else:
                ref_len = ref_subset_len_dict[ref_id]
                if (len(cigar_splitted) == 1) and (cigar[-1] == 'M') and (int(cigar[:-1]) >= minCigarM):
                    ref_pos_end = ref_pos + int(cigar[:-1])

                    # left side: 220M
                    if ref_pos <= max_gap_to_end:
                        qualified_mapping = True
                        # good
                        print(each_line_split)


                    # right side: 116M
                    elif (ref_len - ref_pos_end) <= max_gap_to_end:
                        qualified_mapping = True
                        # good

                if len(cigar_splitted) == 2:

                    # left side: ['6S', '214M']
                    if (cigar_splitted[-1][-1] == 'M') and (cigar_splitted[0][-1] == 'S') and (int(cigar_splitted[-1][:-1]) >= minCigarM) and (ref_pos == 1):
                        qualified_mapping = True
                        # good

                    # right side: ['212M', '8S']
                    elif (cigar_splitted[-0][-1] == 'M') and (cigar_splitted[1][-1] == 'S') and (int(cigar_splitted[0][:-1]) >= minCigarM):
                        if (ref_pos + int(cigar_splitted[0][:-1]) - 1) == ref_len:
                            qualified_mapping = True
                            # good
                # allow one mismatch
                if len(cigar_splitted) == 3:
                    if (cigar_splitted[0][-1] == 'M') and (cigar_splitted[2][-1] == 'M'):
                        left_m_len = int(cigar_splitted[0][:-1])
                        mismatch_len = int(cigar_splitted[1][:-1])
                        right_m_len = int(cigar_splitted[2][:-1])
                        read_len = left_m_len + right_m_len + mismatch_len
                        left_m_pct = left_m_len * 100 / read_len
                        right_m_pct = right_m_len * 100 / read_len
                        if (left_m_len >= 30) and (right_m_len >= 30) and (left_m_pct >= 25) and (right_m_pct >= 25) and (mismatch_len == 1):
                            qualified_mapping = True

            if qualified_mapping is True:
                qualified_reads_to_ref_dict[read_id] = ref_id

                if read_id_base not in qualified_reads_dict:
                    qualified_reads_dict[read_id_base] = [read_strand]
                else:
                    qualified_reads_dict[read_id_base].append(read_strand)

reads_to_extract_to_ref_dict = {}
for qualified_read in qualified_reads_dict:
    read_strand = qualified_reads_dict[qualified_read]
    if len(read_strand) == 1:

        mapped_mate = ''
        mate_to_extract = ''
        if read_strand == ['1']:
            mapped_mate = '%s.1' % (qualified_read)
            mate_to_extract = '%s.2' % (qualified_read)
        if read_strand == ['2']:
            mapped_mate = '%s.2' % (qualified_read)
            mate_to_extract = '%s.1' % (qualified_read)

        if mate_to_extract not in all_mapped_reads:
            reads_to_extract_to_ref_dict[mate_to_extract] = qualified_reads_to_ref_dict[mapped_mate]


print(reads_to_extract_to_ref_dict)
print(len(reads_to_extract_to_ref_dict))

'''
module load java/8u201-jdk
module load bbmap/38.51
cd /srv/scratch/z5039045/MarkerMAG_wd/Kelp/test_cigar_fmt
bbmap.sh ref=test.fasta in=../Kelp_R1.fasta in2=../Kelp_R2.fasta outm=test_1.3.sam local=t nodisk=t ambiguous=all sam=1.3 keepnames=t saa=f silent=true threads=6 -Xmx10g
bbmap.sh ref=test.fasta in=../Kelp_R1.fasta in2=../Kelp_R2.fasta outm=test_1.4.sam local=t nodisk=t ambiguous=all keepnames=t saa=f silent=true threads=6 -Xmx10g



'''


import os
import argparse
from Bio import SeqIO


def sep_path_basename_ext(file_in):
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'
    file_basename, file_extension = os.path.splitext(file_name)
    return file_path, file_basename, file_extension


parser = argparse.ArgumentParser()
parser.add_argument('-ref',   required=True, help='reference')
parser.add_argument('-reads', required=True, help='reads')
args = vars(parser.parse_args())
ref_seq = args['ref']
combined_reads_in = args['reads']


ref_path, ref_basename, ref_extension = sep_path_basename_ext(ref_seq)
reads_in_path, reads_in_basename, reads_in_extension = sep_path_basename_ext(combined_reads_in)
op_r1       = '%s/%s_R1.fa' % (reads_in_path, reads_in_basename)
op_r2       = '%s/%s_R2.fa' % (reads_in_path, reads_in_basename)
op_up       = '%s/%s_UP.fa' % (reads_in_path, reads_in_basename)
ref_index   = '%s/%s'       % (ref_path, ref_basename)
op_sam      = '%s/%s.sam'   % (ref_path, ref_basename)


read_id_set = set()
read_base_set = set()
read_seq_dict = {}
for each_read in SeqIO.parse(combined_reads_in, 'fasta'):
    read_id = each_read.id
    read_base = '.'.join(read_id.split('.')[:-1])
    read_id_set.add(read_id)
    read_base_set.add(read_base)
    read_seq_dict[read_id] = str(each_read.seq)

paired_base_set = set()
unpaired_read_set = set()
for each_base in read_base_set:
    base_r1 = '%s.1' % each_base
    base_r2 = '%s.2' % each_base
    if (base_r1 in read_id_set) and (base_r2 in read_id_set):
        paired_base_set.add(each_base)
    elif (base_r1 in read_id_set) and (base_r2 not in read_id_set):
        unpaired_read_set.add(base_r1)
    elif (base_r1 not in read_id_set) and (base_r2 in read_id_set):
        unpaired_read_set.add(base_r2)

if len(paired_base_set) > 0:
    op_r1_handle = open(op_r1, 'w')
    op_r2_handle = open(op_r2, 'w')
    for each_paired_base in paired_base_set:
        paired_base_r1 = '%s.1' % each_paired_base
        paired_base_r2 = '%s.2' % each_paired_base
        paired_base_r1_seq = read_seq_dict[paired_base_r1]
        paired_base_r2_seq = read_seq_dict[paired_base_r2]
        op_r1_handle.write('>%s\n' % paired_base_r1)
        op_r1_handle.write('%s\n'  % paired_base_r1_seq)
        op_r2_handle.write('>%s\n' % paired_base_r2)
        op_r2_handle.write('%s\n'  % paired_base_r2_seq)
    op_r1_handle.close()
    op_r2_handle.close()

if len(unpaired_read_set) > 0:
    op_up_handle = open(op_up, 'w')
    for each_unpaired_read in unpaired_read_set:
        op_up_handle.write('>%s\n' % each_unpaired_read)
        op_up_handle.write('%s\n'  % read_seq_dict[each_unpaired_read])
    op_up_handle.close()

index_ref_cmd = 'bowtie2-build --quiet -f %s %s' % (ref_seq, ref_index)
bowtie2_cmd = ''
if (len(paired_base_set) > 0) and (len(unpaired_read_set) > 0):
    bowtie2_cmd   = 'bowtie2 -x %s -1 %s -2 %s -U %s -S %s -p 6 -f --local --all --no-unal' % (ref_index, op_r1, op_r2, op_up, op_sam)
if (len(paired_base_set) > 0) and (len(unpaired_read_set) == 0):
    bowtie2_cmd   = 'bowtie2 -x %s -1 %s -2 %s -S %s -p 6 -f --local --all --no-unal' % (ref_index, op_r1, op_r2, op_sam)
if (len(paired_base_set) == 0) and (len(unpaired_read_set) > 0):
    bowtie2_cmd   = 'bowtie2 -x %s -U %s -S %s -p 6 -f --local --all --no-unal' % (ref_index, op_up, op_sam)
if bowtie2_cmd != '':
    os.system('module load bowtie')
    os.system(index_ref_cmd)
    os.system(bowtie2_cmd)
else:
    print('No reads found!')



import os
import glob
import shutil
import argparse
import pandas as pd
import multiprocessing as mp
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from datetime import datetime
from distutils.spawn import find_executable


def get_rc(seq_in):
    seq_in_rc = str(SeqRecord(Seq(seq_in)).reverse_complement().seq)
    return seq_in_rc


def sam_flag_to_rc(flag_value):

    binary_flag = "{0:b}".format(int(flag_value))
    binary_flag_len = len(str(binary_flag))
    binary_flag_polished = '0' * (12 - binary_flag_len) + str(binary_flag)

    read_rced = False
    if binary_flag_polished[7] == '1':
        read_rced = True

    return read_rced


class MappingRecord:

    #  sequences store in r1_seq and r2_seq should NOT been reverse complemented

    def __init__(self):

        self.r1_seq = ''
        self.r2_seq = ''
        self.r1_seq_qual = '*'
        self.r2_seq_qual = '*'

        self.r1_refs = dict()
        self.r2_refs = dict()

        self.r1_cigar_to_flag = dict()
        self.r2_cigar_to_flag = dict()

        self.r1_longest_clp_cigar = ''
        self.r1_longest_clp_falg  = ''
        self.r2_longest_clp_cigar = ''
        self.r2_longest_clp_falg  = ''

        self.qualified_reads           = False
        self.consider_round_2          = False
        self.consider_r1_unmapped_mate = False
        self.consider_r1_clipping_part = False
        self.consider_r2_unmapped_mate = False
        self.consider_r2_clipping_part = False

        self.r1_filtered_refs = set()
        self.r2_filtered_refs = set()

        self.r1_clipping_seq = ''
        self.r2_clipping_seq = ''

        self.r1_clipping_seq_qual = '*'
        self.r2_clipping_seq_qual = '*'

        self.unmapped_r1_refs = set()
        self.unmapped_r2_refs = set()

        self.clipping_r1_refs = set()
        self.clipping_r2_refs = set()


def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if (each_element.isalpha() is True) or (each_element == '='):
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


def get_cigar_stats(cigar_splitted):

    # aligned_len: M I X =
    # clipping_len: S
    # mismatch_len: X I D
    # mismatch_pct = mismatch_len / aligned_len
    # aligned_pct  = aligned_len  / (aligned_len + clipping_len)
    # clipping_pct = clipping_len / (aligned_len + clipping_len)

    aligned_len = 0
    clipping_len = 0
    mismatch_len = 0
    for each_part in cigar_splitted:
        each_part_len = int(each_part[:-1])
        each_part_cate = each_part[-1]

        # get aligned_len
        if each_part_cate in {'M', 'm', 'I', 'i', 'X', 'x', '='}:
            aligned_len += each_part_len

        # get clipping_len
        if each_part_cate in ['S', 's']:
            clipping_len += each_part_len

        # get mismatch_len
        if each_part_cate in {'I', 'i', 'X', 'x', 'D', 'd'}:
            mismatch_len += each_part_len

    aligned_pct  = float("{0:.2f}".format(aligned_len * 100 / (aligned_len + clipping_len)))
    clipping_pct = float("{0:.2f}".format(clipping_len * 100 / (aligned_len + clipping_len)))
    mismatch_pct = float("{0:.2f}".format(mismatch_len * 100 / (aligned_len)))

    return aligned_len, aligned_pct, clipping_len, clipping_pct, mismatch_pct


########################################################################################################################

# file in
pwd_samfile                         = '/Users/songweizhi/Desktop/new_algorithm_Kelp/Kelp_SILVA138_id99_assembled_16S_uclust_0.999.sam'
min_M_pct                           = 20
min_M_len                           = 30
min_clp_len                         = 30
min_clp_M_len                       = 25
max_mis_pct                         = 3

# file out
pwd_samfile_reads                   = '/Users/songweizhi/Desktop/new_algorithm_Kelp/Kelp_SILVA138_id99_assembled_16S_uclust_0.999_reads.fa'
clipping_part_seq                   = '/Users/songweizhi/Desktop/new_algorithm_Kelp/clipping_parts.fa'
unmapped_mates                      = '/Users/songweizhi/Desktop/new_algorithm_Kelp/unmapped_mates.fa'
clipping_part_seq_sam               = '/Users/songweizhi/Desktop/new_algorithm_Kelp/clipping_parts.sam'
unmapped_mates_sam                  = '/Users/songweizhi/Desktop/new_algorithm_Kelp/unmapped_mates.sam'
stats_combined                      = '/Users/songweizhi/Desktop/new_algorithm_Kelp/stats_combined.txt'
stats_combined_filtered             = '/Users/songweizhi/Desktop/new_algorithm_Kelp/stats_combined_filtered.txt'
blast_results_all_vs_all_16s        = '/Users/songweizhi/Desktop/new_algorithm_Kelp/16S_all_vs_all_blastn.tab'
mock_final_op                       = '/Users/songweizhi/Desktop/new_algorithm_Kelp/mock_final_op.txt'
mock_final_op_ctg_level             = '/Users/songweizhi/Desktop/new_algorithm_Kelp/mock_final_op_ctg_level.txt'

marker_to_ctg_Key_connector_str = '___M___'
marker_to_gnm_Key_connector_str = '___M___'
gnm_ctg_connector = '___'


########################################################################################################################

MappingRecord_dict = {}
for each_read in open(pwd_samfile):
    if not each_read.startswith('@'):
        store_read_seq = False
        each_read_split = each_read.strip().split('\t')
        read_id = each_read_split[0]
        read_id_base = '.'.join(read_id.split('.')[:-1])
        read_strand = read_id.split('.')[-1]
        read_flag = int(each_read_split[1])
        read_seq = each_read_split[9]
        read_seq_qual = each_read_split[10]
        cigar = each_read_split[5]

        if cigar != '*':
            ref_id = each_read_split[2]
            ref_pos = each_read_split[3]
            ref_id_with_pos = '%s_pos_%s' % (ref_id, ref_pos)
            cigar_splitted = cigar_splitter(cigar)
            aligned_len, aligned_pct, clipping_len, clipping_pct, mismatch_pct = get_cigar_stats(cigar_splitted)

            if (aligned_len >= min_M_len) and (aligned_pct >= min_M_pct) and (mismatch_pct <= (max_mis_pct)):
                if read_id_base not in MappingRecord_dict:
                    MappingRecord_dict[read_id_base] = MappingRecord()
                if read_strand == '1':
                    MappingRecord_dict[read_id_base].r1_refs[ref_id_with_pos] = cigar
                    MappingRecord_dict[read_id_base].r1_cigar_to_flag[cigar] = read_flag
                    store_read_seq = True
                if read_strand == '2':
                    MappingRecord_dict[read_id_base].r2_refs[ref_id_with_pos] = cigar
                    MappingRecord_dict[read_id_base].r2_cigar_to_flag[cigar] = read_flag
                    store_read_seq = True




            else:
                store_read_seq = True
        else:
            store_read_seq = True

        # store_read_seq into dict
        if store_read_seq is True:
            read_rc = sam_flag_to_rc(read_flag)

            if read_id_base not in MappingRecord_dict:
                MappingRecord_dict[read_id_base] = MappingRecord()

            # turn back if read reverse complemented
            read_seq_to_store = read_seq
            read_seq_qual_to_store = read_seq_qual
            if read_rc is True:
                read_seq_to_store = get_rc(read_seq)
                read_seq_qual_to_store = read_seq_qual[::-1]

            if read_strand == '1':
                MappingRecord_dict[read_id_base].r1_seq = read_seq_to_store
                MappingRecord_dict[read_id_base].r1_seq_qual = read_seq_qual_to_store
            if read_strand == '2':
                MappingRecord_dict[read_id_base].r2_seq = read_seq_to_store
                MappingRecord_dict[read_id_base].r2_seq_qual = read_seq_qual_to_store


        print(each_read_split)
        print(MappingRecord_dict[read_id_base].r1_cigar_to_flag)
        print(MappingRecord_dict[read_id_base].r2_cigar_to_flag)
        print()





def blast_results_to_pairwise_16s_iden_dict(blastn_output, align_len_cutoff, cov_cutoff):
    pairwise_iden_dict = {}
    for match in open(blastn_output):
        match_split = match.strip().split('\t')
        query = match_split[0]
        subject = match_split[1]
        iden = float(match_split[2])
        align_len = int(match_split[3])
        query_len = int(match_split[12])
        subject_len = int(match_split[13])
        coverage_q = float(align_len) * 100 / float(query_len)
        coverage_s = float(align_len) * 100 / float(subject_len)

        if (align_len >= align_len_cutoff) and (query != subject) and (coverage_q >= cov_cutoff) and (
                coverage_s >= cov_cutoff):
            query_to_subject_key = '__|__'.join(sorted([query, subject]))
            if query_to_subject_key not in pairwise_iden_dict:
                pairwise_iden_dict[query_to_subject_key] = iden
            else:
                if iden > pairwise_iden_dict[query_to_subject_key]:
                    pairwise_iden_dict[query_to_subject_key] = iden

    return pairwise_iden_dict


blast_results_all_vs_all_16s = '/Users/songweizhi/Desktop/GI_0524_128_60_60_16S_all_vs_all_blastn.tab'
min_aln_16s = 500
min_cov_16s = 30

pairwise_16s_iden_dict = blast_results_to_pairwise_16s_iden_dict(blast_results_all_vs_all_16s, min_aln_16s, min_cov_16s)
highest_link_16s_list = ['3_GI_subsample_100_1336', '3_GI_subsample_75_2502', '3_GI_subsample_25_763', '3_GI_subsample_100_2066', '3_GI_subsample_75_69', '3_GI_subsample_100_3364', '3_GI_subsample_75_1008', '3_GI_subsample_25_565',
                         '3_GI_subsample_25_345', '3_GI_subsample_10_173', '3_GI_subsample_75_988',
                         '3_GI_subsample_75_1644']


def get_mean_iden_list(linked_16s_list):

    mean_iden_list = []
    for each_16s_1 in linked_16s_list:
        current_16s_iden_list = []
        for each_16s_2 in linked_16s_list:
            if each_16s_1 != each_16s_2:
                key = '__|__'.join(sorted([each_16s_1, each_16s_2]))
                key_iden = pairwise_16s_iden_dict.get(key, 'na')
                if key_iden != 'na':
                    current_16s_iden_list.append(key_iden)
        mean_iden = sum(current_16s_iden_list) / len(current_16s_iden_list)
        mean_iden = float("{0:.3f}".format(mean_iden))
        # print('%s\t%s\t%s' % (each_16s_1, mean_iden, current_16s_iden_list))
        mean_iden_list.append(mean_iden)

    return mean_iden_list


mean_iden_list = get_mean_iden_list(highest_link_16s_list)

sorted_seq_id_list = [seq_id for mean_iden, seq_id in sorted(zip(mean_iden_list, highest_link_16s_list), reverse=True)]

print(highest_link_16s_list)

print(sorted(mean_iden_list, reverse=True))

print(sorted_seq_id_list)


import os
from Bio import SeqIO


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def overlap_between_list(list_1, list_2):

    overlap = False
    for list_1_element in list_1:
        if list_1_element in list_2:
            overlap = True

    return overlap


########################################################################################################################

wd = '/Users/songweizhi/Desktop/assess_linkages'

########## reference to cluster ##########

drep_ani_cutoff             = 95
drep_cdb_file               = '%s/file_in/Cdb_%s.csv'                               % (wd, drep_ani_cutoff)
ref_to_strain_file          = '%s/file_in/ref_to_strain.txt'                        % wd

########## bin to reference ##########

parse_blastn_bin_vs_ref     = False  # True or False
blastn_bin_vs_ref           = '%s/file_in/bin_vs_ref.tab'                           % wd
iden_cutoff                 = 99.9
aln_len_cutoff              = 1500
cov_q_cutoff                = 90
min_match_length            = 102400  # 100 Kbp

########## 16S to reference ##########

perform_blastn_16s_vs_refs  = False  # True or False
combined_GI_ref_16S         = '%s/file_in/combined_GI_ref_16S.ffn'                          % wd
matam_16s_seqs              = '%s/file_in/3_GI_assembled_16S_uclust_0.995.fasta'            % wd
matam_16s_blastn            = '%s/file_in/3_GI_assembled_16S_uclust_0.995.fasta.blastn'     % wd
iden_cutoff_16s             = 99.3  # 99.3 (best), 99.5
aln_len_cutoff_16s          = 500
cov_q_cutoff_16s            = 70
total_query_mag_num         = 97

########## assessment results ##########

MarkerMAG_linkages          = '%s/file_in/MarkerMAG_Mita_test_combined_linkages.txt'                  % wd
# Linkage	51/97(52.58)	51/52(98.08)	10

MarkerMAG_linkages          = '%s/file_in/GI_combined_linkages.txt'                  % wd


########## script ##########

pwd_plot_sankey_R = '%s/file_in/get_sankey_plot.R' % wd

############################################### define file/folder name ################################################

# bin to reference
bin_vs_ref_txt              = '%s/bin_vs_ref_imag%s.txt'                    % (wd, iden_cutoff)
stats_bin_to_ref_txt        = '%s/stats_bin_to_ref_imag%s.txt'              % (wd, iden_cutoff)
stats_ref_to_bin_txt        = '%s/stats_ref_to_bin_imag%s.txt'              % (wd, iden_cutoff)
stats_bin_to_cluster_txt    = '%s/stats_bin_to_cluster_ani%s_imag%s.txt'    % (wd, drep_ani_cutoff, iden_cutoff)
stats_cluster_to_bin_txt    = '%s/stats_cluster_to_bin_ani%s_imag%s.txt'    % (wd, drep_ani_cutoff, iden_cutoff)

# assessment results
linkage_file_path, linkage_file_basename, linkage_file_extension = sep_path_basename_ext(MarkerMAG_linkages)
MarkerMAG_linkages_assessed = '%s/%s_with_assessment_ani%s_imag%s_i16S%s.txt'   % (wd, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)
wrong_linkages_txt          = '%s/%s_wrong_ani%s_imag%s_i16S%s.txt'             % (wd, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)
unknown_linkages_txt        = '%s/%s_unknown_ani%s_imag%s_i16S%s.txt'           % (wd, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)


################################################# reference to cluster #################################################

cluster_to_ref_dict = {}
ref_to_cluster_dict = {}
for each_ref in open(drep_cdb_file):
    if not each_ref.startswith('genome,secondary_cluster'):
        each_ref_split = each_ref.strip().split(',')
        ref_file_name = each_ref_split[0]
        ref_file_name_no_ext = '.'.join(ref_file_name.split('.')[:-1])
        ref_cluster = 'C' + each_ref_split[1]
        ref_to_cluster_dict[ref_file_name_no_ext] = ref_cluster
        if ref_cluster not in cluster_to_ref_dict:
            cluster_to_ref_dict[ref_cluster] = [ref_file_name_no_ext]
        else:
            cluster_to_ref_dict[ref_cluster].append(ref_file_name_no_ext)


################################################### bin to reference ###################################################

bin_ref_connector           = '__|__'

# get ref_to_strain_dict
ref_to_strain_dict = {}
for ref in open(ref_to_strain_file):
    ref_split = ref.strip().split('\t')
    ref_to_strain_dict[ref_split[0]] = ref_split[1]

if parse_blastn_bin_vs_ref is True:
    bin_vs_ref_dict = {}
    for match in open(blastn_bin_vs_ref):
        match_split = match.strip().split('\t')
        query = match_split[0]
        query_genome = '_'.join(query.split('_')[:2])
        subject = match_split[1]
        subject_genome = '_'.join(subject.split('_')[:2])
        iden = float(match_split[2])
        aln_len = int(match_split[3])
        query_len = int(match_split[12])
        subject_len = int(match_split[13])
        coverage_q = float(aln_len) * 100 / float(query_len)
        if (iden >= iden_cutoff) and (aln_len > aln_len_cutoff) and (coverage_q >= cov_q_cutoff):
            key_bin_ref = '%s%s%s' % (query_genome, bin_ref_connector, subject_genome)
            if key_bin_ref not in bin_vs_ref_dict:
                bin_vs_ref_dict[key_bin_ref] = aln_len
            else:
                bin_vs_ref_dict[key_bin_ref] += aln_len

    bin_vs_ref_dict_filtered = {}
    for each_key in bin_vs_ref_dict:
        if bin_vs_ref_dict[each_key] >= min_match_length:
            bin_vs_ref_dict_filtered[each_key] = bin_vs_ref_dict[each_key]

    # write out linkages
    linkage_txt_handle = open(bin_vs_ref_txt, 'w')
    linkage_txt_handle.write('Bin,Ref,Length\n')
    bin_to_ref_dict = {}
    ref_to_bin_dict = {}
    bin_to_cluster_dict = {}
    cluster_to_bin_dict = {}
    for each_linkage in bin_vs_ref_dict_filtered:
        linkage_split = each_linkage.split(bin_ref_connector)
        bin_id = linkage_split[0]
        ref_id = linkage_split[1]
        ref_cluster = ref_to_cluster_dict[ref_id]

        if bin_id not in bin_to_ref_dict:
            bin_to_ref_dict[bin_id] = {ref_id}
        else:
            bin_to_ref_dict[bin_id].add(ref_id)

        if bin_id not in bin_to_cluster_dict:
            bin_to_cluster_dict[bin_id] = {ref_cluster}
        else:
            bin_to_cluster_dict[bin_id].add(ref_cluster)

        if ref_id not in ref_to_bin_dict:
            ref_to_bin_dict[ref_id] = {bin_id}
        else:
            ref_to_bin_dict[ref_id].add(bin_id)

        if ref_cluster not in cluster_to_bin_dict:
            cluster_to_bin_dict[ref_cluster] = {bin_id}
        else:
            cluster_to_bin_dict[ref_cluster].add(bin_id)

        linkage_txt_handle.write('%s,%s,%s\n' % (bin_id, ref_id, bin_vs_ref_dict_filtered[each_linkage]))
    linkage_txt_handle.close()

    # visualize
    cmd_sankey_bin_to_ref = 'Rscript %s -f %s -x %s -y %s' % (pwd_plot_sankey_R, bin_vs_ref_txt, 600, 1800)
    os.system(cmd_sankey_bin_to_ref)

    bin_to_ref_txt_handle = open(stats_bin_to_ref_txt, 'w')
    for each_bin in bin_to_ref_dict:
        bin_to_ref_txt_handle.write('%s\t%s\n' % (each_bin, ','.join(bin_to_ref_dict[each_bin])))
    bin_to_ref_txt_handle.close()

    ref_to_bin_txt_handle = open(stats_ref_to_bin_txt, 'w')
    for each_ref in ref_to_bin_dict:
        ref_to_bin_txt_handle.write('%s\t%s\t%s\n' % (each_ref, ','.join(ref_to_bin_dict[each_ref]), ref_to_strain_dict[each_ref]))
    ref_to_bin_txt_handle.close()

    stats_bin_to_cluster_txt_handle = open(stats_bin_to_cluster_txt, 'w')
    for each_bin in bin_to_cluster_dict:
        stats_bin_to_cluster_txt_handle.write('%s\t%s\n' % (each_bin, ','.join(bin_to_cluster_dict[each_bin])))
    stats_bin_to_cluster_txt_handle.close()

    stats_cluster_to_bin_txt_handle = open(stats_cluster_to_bin_txt, 'w')
    for each_ref in cluster_to_bin_dict:
        stats_cluster_to_bin_txt_handle.write('%s\t%s\n' % (each_ref, ','.join(cluster_to_bin_dict[each_ref])))
    stats_cluster_to_bin_txt_handle.close()

bin_to_cluster_dict = {}
for each_match in open(stats_bin_to_cluster_txt):
    each_match_split = each_match.strip().split('\t')
    bin_to_cluster_dict[each_match_split[0]] = {i for i in each_match_split[1].split(',')}


################################################### 16S to reference ###################################################

blast_parameters = '-evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 1'
blast_cmd = 'blastn -query %s -subject %s -out %s %s' % (matam_16s_seqs, combined_GI_ref_16S, matam_16s_blastn, blast_parameters)
if perform_blastn_16s_vs_refs is True:
    os.system(blast_cmd)

# get matam_16s_to_cluster_dict
matam_16s_to_cluster_dict = {}
for match in open(matam_16s_blastn):
    match_split = match.strip().split('\t')
    query = match_split[0]
    subject = match_split[1]
    subject_gnm = subject.split('_16S_')[0]
    subject_cluster = ref_to_cluster_dict[subject_gnm]
    iden = float(match_split[2])
    aln_len = int(match_split[3])
    query_len = int(match_split[12])
    subject_len = int(match_split[13])
    coverage_q = float(aln_len) * 100 / float(query_len)
    if (iden >= iden_cutoff_16s) and (aln_len > aln_len_cutoff_16s) and (coverage_q >= cov_q_cutoff_16s):
        if query not in matam_16s_to_cluster_dict:
            matam_16s_to_cluster_dict[query] = {subject_cluster}
        else:
            matam_16s_to_cluster_dict[query].add(subject_cluster)


###################################################### assessment ######################################################

MarkerMAG_linkages_assessed_handle = open(MarkerMAG_linkages_assessed, 'w')
wrong_linkages_txt_handle = open(wrong_linkages_txt, 'w')
unknown_linkages_txt_handle = open(unknown_linkages_txt, 'w')
linkage_num_right = 0
linkage_num_wrong = 0
linkage_num_unknown = 0
for each_linkage in open(MarkerMAG_linkages):
    if each_linkage.startswith('MarkerGene\tGenomicSeq\tLinkage\tStep'):
        MarkerMAG_linkages_assessed_handle.write('MarkerGene\tGenomicSeq\tLinkage\tStep\tAssessment\n')
    else:
        each_linkage_split = each_linkage.strip().split('\t')
        id_16s = each_linkage_split[0]
        id_mag = each_linkage_split[1]

        matched_cluster_16s = {}
        if id_16s in matam_16s_to_cluster_dict:
            matched_cluster_16s = matam_16s_to_cluster_dict[id_16s]

        matched_cluster_mag = {}
        if id_mag in bin_to_cluster_dict:
            matched_cluster_mag = bin_to_cluster_dict[id_mag]

        if (matched_cluster_16s != {}) and (matched_cluster_mag != {}):

            if overlap_between_list(matched_cluster_mag, matched_cluster_16s) is True:
                linkage_num_right += 1
                MarkerMAG_linkages_assessed_handle.write('%s\tCorrect\n' % each_linkage.strip())
            else:
                linkage_num_wrong += 1
                MarkerMAG_linkages_assessed_handle.write('%s\tWrong\n' % each_linkage.strip())
                for each_16s_cluster in matched_cluster_16s:
                    current_16s_cluster_ref = cluster_to_ref_dict[each_16s_cluster]
                    for each_ref_1 in current_16s_cluster_ref:
                        wrong_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_wrong, id_16s, id_mag, '16S', each_16s_cluster, each_ref_1, ref_to_strain_dict[each_ref_1]))

                for each_mag_cluster in matched_cluster_mag:
                    current_mag_cluster_ref = cluster_to_ref_dict[each_mag_cluster]
                    for each_ref_2 in current_mag_cluster_ref:
                        wrong_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_wrong, id_16s, id_mag, 'MAG', each_mag_cluster, each_ref_2, ref_to_strain_dict[each_ref_2]))
                wrong_linkages_txt_handle.write('\n')
        else:
            linkage_num_unknown += 1
            MarkerMAG_linkages_assessed_handle.write('%s\tUnknown\n' % each_linkage.strip())
            for each_16s_cluster in matched_cluster_16s:
                current_16s_cluster_ref = cluster_to_ref_dict[each_16s_cluster]
                for each_ref_1 in current_16s_cluster_ref:
                    unknown_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_unknown, id_16s, id_mag, '16S', each_16s_cluster, each_ref_1, ref_to_strain_dict[each_ref_1]))
            for each_mag_cluster in matched_cluster_mag:
                current_mag_cluster_ref = cluster_to_ref_dict[each_mag_cluster]
                for each_ref_2 in current_mag_cluster_ref:
                    unknown_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_unknown, id_16s, id_mag, 'MAG', each_mag_cluster, each_ref_2, ref_to_strain_dict[each_ref_2]))
            unknown_linkages_txt_handle.write('\n')
wrong_linkages_txt_handle.close()
unknown_linkages_txt_handle.close()
MarkerMAG_linkages_assessed_handle.close()

# report
link_recovery = linkage_num_right*100/total_query_mag_num
link_accuracy = linkage_num_right*100/(linkage_num_right + linkage_num_wrong)
link_recovery = float("{0:.2f}".format(link_recovery))
link_accuracy = float("{0:.2f}".format(link_accuracy))
recovery_str  = '%s/%s(%s)' % (linkage_num_right, total_query_mag_num, link_recovery)
accuracy_str  = '%s/%s(%s)' % (linkage_num_right, (linkage_num_right + linkage_num_wrong), link_accuracy)

print('%s\tRecovery\tAccuracy\tUnknown' % 'Linkage')
print('%s\t%s\t%s\t%s' % ('Linkage', recovery_str, accuracy_str, linkage_num_unknown))


########################################################################################################################

'''
MarkerMAG_linkages          = '%s/file_in/CAMI2_GI_mplu5_longkmer_33-127_combined_linkages.txt'                  % wd
MarkerMAG_linkages          = '%s/file_in/CAMI2_GI_mplu5_longkmer_55-127_combined_linkages.txt'                  % wd
MarkerMAG_linkages          = '%s/file_in/CAMI2_GI_mplu5_9_combined_linkages.txt'                  % wd
MarkerMAG_linkages          = '%s/file_in/CAMI2_GI_mplu5_longkmer_combined_linkages.txt'                  % wd
MarkerMAG_linkages          = '%s/file_in/CAMI2_GI_mplu5_combined_linkages.txt'                  % wd

MarkerMAG_linkages          = '%s/file_in/MarkerMAG_Mita_test_combined_linkages.txt'                  % wd
# Linkage	51/97(52.58)	51/52(98.08)	10

MarkerMAG_linkages          = '%s/file_in/MarkerMAG_Mira_by_mapping_cigarMpct50_combined_linkages.txt'                  % wd
# Linkage	61/97(62.89)	61/68(89.71)	21
# Linkage	60/97(61.86)	60/67(89.55)	20
# Linkage	69/97(71.13)	69/76(90.79)	11

MarkerMAG_linkages          = '%s/file_in/MarkerMAG_Mira_by_mapping_cigarMpct50_mismatch0.55_combined_linkages.txt'                  % wd
# 99.5  Linkage	60/97(61.86)	60/67(89.55)	19
# 99.3  Linkage	68/97(70.1)	    68/75(90.67)	11
# 99.0  Linkage	72/97(74.23)	72/79(91.14)	7

MarkerMAG_linkages          = '%s/file_in/MarkerMAG_Mira_by_mapping_cigarMpct50_mismatch0.55_mplu10_combined_linkages.txt'                  % wd
# Linkage	55/97(56.7)	55/62(88.71)	11
# Linkage	57/97(58.76)	57/64(89.06)	9
'''


import random


genome_id_list = ['s1', 's2', 's3', 's4', 's5', 'g1', 'g2', 'g3', 'g4', 'g5', 'f1', 'f2', 'f3', 'f4', 'f5', 'o1', 'o2', 'o3', 'o4', 'o5', 'c1', 'c2', 'c3', 'p1', 'p2', 'p3', 'p4', 'p5']

# 25x       47.58
# 1x        1.903
# 8.33x     15.86 x 28 = 444


random_lol = []
for i in range(0,3):
    randomlist = random.sample(range(1, 100), 28)
    randomlist_sum = sum(randomlist)
    divid_factor = randomlist_sum/444
    randomlist_norm = [round(i/divid_factor) for i in randomlist]

    depth_file = '/Users/songweizhi/Desktop/depth_rep%s.txt' % (i + 1)
    depth_file_handle = open(depth_file, 'w')
    for (genome, depth) in zip(genome_id_list, randomlist_norm):
        depth_file_handle.write('%s\t%s\n' % (genome, depth))
    depth_file_handle.close()

    print(randomlist_norm)



from Bio import SeqIO


ref_in      = '/Users/songweizhi/Desktop/000/BH_ER_050417_refined_bins_combined.fa'
ref_subset  = '/Users/songweizhi/Desktop/000/BH_ER_050417_refined_bins_combined_subset.fa'
end_seq_len = 1800


# get ref seqs subset
ref_subset_len_dict = {}
ref_subset_handle = open(ref_subset, 'w')
for ref_seq in SeqIO.parse(ref_in, 'fasta'):
    ref_seq_id = ref_seq.id
    ref_seq_len = len(ref_seq.seq)
    if ref_seq_len < end_seq_len * 3:
        ref_subset_handle.write('>%s\n' % ref_seq_id)
        ref_subset_handle.write('%s\n' % ref_seq.seq)
        ref_subset_len_dict[ref_seq_id] = ref_seq_len
    else:
        ref_seq_left_end_id = '%s_l' % ref_seq_id
        ref_seq_right_end_id = '%s_r' % ref_seq_id
        ref_seq_left_end = ref_seq.seq[:end_seq_len]
        ref_seq_right_end = ref_seq.seq[-end_seq_len:]

        # write out left end
        ref_subset_handle.write('>%s\n' % ref_seq_left_end_id)
        ref_subset_handle.write('%s\n' % ref_seq_left_end)

        # write out right end
        ref_subset_handle.write('>%s\n' % ref_seq_right_end_id)
        ref_subset_handle.write('%s\n' % ref_seq_right_end)
ref_subset_handle.close()





import os
import glob


mag_folder    = '/Users/songweizhi/Desktop/Refined_refined_bins_renamed'
mag_ext       = 'fna'
combined_16s  = '/Users/songweizhi/Desktop/combined_16S.ffn'
combined_mags = '/Users/songweizhi/Desktop/combined_MAGs.fna'


# rename ctgs in mags
mag_file_re = '%s/*.%s' % (mag_folder, mag_ext)
mag_file_list = [os.path.basename(file_name) for file_name in glob.glob(mag_file_re)]
for mag_file in mag_file_list:
    mag_file_no_ext = '.'.join(mag_file.split('.')[:-1])
    rename_cmd = 'BioSAK rename_seq -in %s/%s -prefix %s' % (mag_folder, mag_file, mag_file_no_ext)
    os.system(rename_cmd)


# combine renamed mags
combine_cmd = 'cat %s/*_renamed.fna > %s' % (mag_folder, combined_mags)
remove_renamed_mags_cmd = 'rm %s/*_renamed.fna' % mag_folder
os.system(combine_cmd)
os.system(remove_renamed_mags_cmd)







#!/usr/bin/env python3

import os
import argparse
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC
from Bio.SeqRecord import SeqRecord


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


# file in
rm_16s_parser = argparse.ArgumentParser(description='', usage='')
rm_16s_parser.add_argument('-gbk', required=True, type=str, help='gbk file')
args = vars(rm_16s_parser.parse_args())
gbk_file = args['gbk']


# define file name
gbk_file_path, gbk_file_basename, gbk_file_extension = sep_path_basename_ext(gbk_file)
fasta_out = '%s/%s_no_16S.fasta' % (gbk_file_path, gbk_file_basename)


pos_list_16s = [1]
genome_seq_str = ''
for record in SeqIO.parse(gbk_file, 'genbank'):
    genome_seq_str = str(record.seq)
    for gene in record.features:
        if 'locus_tag' in gene.qualifiers:
            if gene.qualifiers['product'] == ['16S ribosomal RNA']:
                start_pos = gene.location.start
                end_pos = gene.location.end
                pos_list_16s.append(int(start_pos))
                pos_list_16s.append(int(end_pos))

pos_list_16s.append(len(genome_seq_str))

pos_list_16s = sorted(pos_list_16s)

fasta_out_handle = open(fasta_out, 'w')
n = 1
while n <= len(pos_list_16s):

    between_region_index = n//2 + 1

    if pos_list_16s[n-1] == 1:
        between_region_start = 1
    else:
        between_region_start = pos_list_16s[n-1] + 1

    if pos_list_16s[n] == len(genome_seq_str):
        between_region_end = len(genome_seq_str)
    else:
        between_region_end = pos_list_16s[n] - 1

    between_region_seq     = genome_seq_str[between_region_start - 1: between_region_end]
    between_region_seq_id  = '%s_%s'   % (gbk_file_basename, between_region_index)
    between_region_seq_des = '%s-%sbp' % (between_region_start, between_region_end)

    export_dna_record(between_region_seq, between_region_seq_id, between_region_seq_des, fasta_out_handle)

    n += 2

fasta_out_handle.close()


'''

cd /Users/songweizhi/Desktop
python3 ~/PycharmProjects/MarkerMAG/script_backup/rm_16S.py -gbk c3.gbk

module load python/3.7.3
cd /srv/scratch/z5039045/MarkerMAG_wd/genome_selection_3/gbk_files
python3 /srv/scratch/z5039045/MarkerMAG_wd/rm_16S.py c1.gbk
python3 /srv/scratch/z5039045/MarkerMAG_wd/rm_16S.py c2.gbk
python3 /srv/scratch/z5039045/MarkerMAG_wd/rm_16S.py c3.gbk

'''


import os
from Bio import SeqIO


###################################################### file in/out #####################################################

wd = '/Users/songweizhi/Desktop/ttt_dRep97'

# file in
drep_cdb_file               = '%s/Cdb.csv'                                      % wd
combined_GI_ref_16S         = '%s/combined_GI_ref_16S.ffn'                      % wd
matam_16s_seqs              = '%s/3_GI_assembled_16S_uclust_0.995.fasta'        % wd
iden_cutoff                 = 99.5
iden_cutoff                 = 100
aln_len_cutoff              = 500
cov_q_cutoff                = 90

# file out
r16s_to_cluster_file        = '%s/16S_to_cluster.txt'                           % wd
rename_16s_file             = '%s/16S_rename.txt'                               % wd
combined_GI_ref_16S_renamed = '%s/combined_GI_ref_16S_renamed.ffn'              % wd

# tmp files
matam_16s_blastn            = '%s/3_GI_assembled_16S_uclust_0.995_vs_refs.tab'  % wd


########################################################################################################################

# get new cluster id
cluster_id_default_to_new_dict = {}
cluster_id_index = 1
for each_mag in open(drep_cdb_file):
    if not each_mag.startswith('genome,secondary_cluster'):
        mag_cluster = each_mag.strip().split(',')[1]
        if mag_cluster not in cluster_id_default_to_new_dict:
            new_id = 'C%s' % cluster_id_index
            cluster_id_index += 1
            cluster_id_default_to_new_dict[mag_cluster] = new_id


cluster_to_mag_dict = {}
mag_to_cluster_dict = {}
for each_mag in open(drep_cdb_file):
    if not each_mag.startswith('genome,secondary_cluster'):
        each_mag_split = each_mag.strip().split(',')
        mag_file_name = each_mag_split[0]
        mag_file_name_no_ext = '.'.join(mag_file_name.split('.')[:-1])
        mag_cluster_default = each_mag_split[1]
        mag_cluster_new = cluster_id_default_to_new_dict[mag_cluster_default]
        mag_to_cluster_dict[mag_file_name_no_ext] = mag_cluster_new
        if mag_cluster_new not in cluster_to_mag_dict:
            cluster_to_mag_dict[mag_cluster_new] = [mag_file_name_no_ext]
        else:
            cluster_to_mag_dict[mag_cluster_new].append(mag_file_name_no_ext)


r16s_to_cluster_file_handle = open(r16s_to_cluster_file, 'w')
s16_to_cluster_dict = {}
cluster_to_s16_dict = {}
for record_16s in SeqIO.parse(combined_GI_ref_16S, 'fasta'):
    s16_genome = record_16s.id.split('_16S_')[0]
    s16_genome_cluster = mag_to_cluster_dict[s16_genome]
    r16s_to_cluster_file_handle.write('%s\t%s\n' % (record_16s.id, s16_genome_cluster))
    s16_to_cluster_dict[record_16s.id] = s16_genome_cluster
    if s16_genome_cluster not in cluster_to_s16_dict:
        cluster_to_s16_dict[s16_genome_cluster] = [record_16s.id]
    else:
        cluster_to_s16_dict[s16_genome_cluster].append(record_16s.id)
r16s_to_cluster_file_handle.close()


print('cluster_id_default_to_new_dict')
print(cluster_id_default_to_new_dict)
print()
print('cluster_to_mag_dict')
print(cluster_to_mag_dict)
print()
print('mag_to_cluster_dict')
print(mag_to_cluster_dict)
print()
print('cluster_to_s16_dict')
print(cluster_to_s16_dict)
print()
print('s16_to_cluster_dict')
print(s16_to_cluster_dict)
print()


blast_parameters = '-evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 1'
blast_cmd = 'blastn -query %s -subject %s -out %s %s' % (matam_16s_seqs, combined_GI_ref_16S, matam_16s_blastn, blast_parameters)
#os.system(blast_cmd)

# parse blast results
matam_16s_to_cluster_dict = {}
for match in open(matam_16s_blastn):
    match_split = match.strip().split('\t')
    query = match_split[0]
    subject = match_split[1]
    iden = float(match_split[2])
    aln_len = int(match_split[3])
    query_len = int(match_split[12])
    subject_len = int(match_split[13])
    coverage_q = float(aln_len) * 100 / float(query_len)
    if (iden >= iden_cutoff) and (aln_len > aln_len_cutoff) and (coverage_q >= cov_q_cutoff):
        subject_cluster = s16_to_cluster_dict[subject]
        if query not in matam_16s_to_cluster_dict:
            matam_16s_to_cluster_dict[query] = {subject_cluster}
        else:
            matam_16s_to_cluster_dict[query].add(subject_cluster)

m = 0
n = 0
for matam_16s in matam_16s_to_cluster_dict:
    matched_clusters = matam_16s_to_cluster_dict[matam_16s]
    if len(matched_clusters) == 1:
        m += 1
    if len(matched_clusters) > 1:
        n += 1

        print('%s\t%s' % (matam_16s, matched_clusters))

print(m)
print(n)

# C63   d__Bacteria; p__Firmicutes; c__Bacilli; o__Bacillales; f__Bacillaceae; g__Bacillus; s__Bacillus subtilis
'''
C58: ['OTU_97.1969.0']  195103
C59: ['OTU_97.38225.0'] 1502
C60: ['OTU_97.11107.0'] 1502
C61: ['OTU_97.9303.0']  195102
C62: ['OTU_97.29691.0'] 1502
C63: ['OTU_97.207.0']   289380
C64: ['OTU_97.31703.1'] 1502
'''

